* INDEX.CLS
* Example CLASS module for alternate key index scan
* Copyright (c) 2008 Ladybridge Systems, All Rights Reserved
*
* START-HISTORY:
* 26 Sep 08  2.8-3 New program.
* END-HISTORY
*
* This class module provides an object that allows an application to walk
* through an alternate key index record by record instead of using
* SELECTLEFT and SELECTRIGHT which return all record ids for a single indexed
* value in one operation.
*
* Because this class module reads the index internally using SELECTLEFT and
* SELECTRIGHT, changes to the file that affect the indexed value that is
* currently cached here will not be reflected in the returned data.
*
* To use this class module, a program must first instantiate the object using
*   OBJ = OBJECT("!INDEX.CLS", FVAR, INDEX.NAME)
* where
*   OBJ is the variable to represent the instantiated object.
*   FVAR is the file variable for the file being scanned.
*   INDEX.NAME is the name of the alternate key index.
* The scan position will be set at the extreme left of the index.
*
* To fetch the next record in either direction, use
*   ID = OBJ->NEXT
* or
*   ID = OBJ->PREV
* The OBJ->KEY property will return the key value associated with the record.
*
* To reposition at the extreme edge of the index, use
*   OBJ->LEFT
* or
*   OBJ->RIGHT
*
* To reposition at the first record with a specified indexed value, use
*   OBJ->SET(VALUE)
* If there is no such item, the index will be positioned to where this value
* would be placed. Use of the LEFT or RIGHT methods will move in the specified
* direction from this point.
*
* Note that this class module uses select list 10 internally.


class index.cls
$catalog !index.cls global

   private fvar, index.name             ;* File variable and index name
   private items, num.items, item.index ;* Item cache
   public key readonly

* ======================================================================
* CREATE.OBJECT - Intialise at left of index

   public subroutine create.object(fvar.arg, index.name.arg)
      fvar = fvar.arg
      index.name = index.name.arg
      items = ''
      num.items = 0
      item.index = 0
      setleft index.name from fvar
   end

* ======================================================================
* LEFT - Position at left of index

   public subroutine left
      items = ''
      num.items = 0
      item.index = 0
      setleft index.name from fvar
   end

* ======================================================================
* NEXT - Fetch next record id

   public function next
      if item.index >= num.items then
         *  Fetch next index entry to the right
         selectright index.name from fvar setting key to 10
         readlist items from 10 else null
         num.items = dcount(items, @fm)
         item.index = 0
         if num.items = 0 then return ''
      end

      item.index += 1
      id = items<item.index>
      return id
   end

* ======================================================================
* PREV - Fetch previous record id

   public function prev
      if item.index <= 1 then
         *  Fetch next index entry to the left
         selectleft index.name from fvar setting key to 10
         readlist items from 10 else null
         num.items = dcount(items, @fm)
         item.index = num.items + 1
         if num.items = 0 then return ''
      end

      item.index -= 1
      id = items<item.index>
      return id
   end

* ======================================================================
* SET - Position at first record with specific indexed value

   public subroutine set(value)
      key = value
      selectindex index.name, key from fvar to 10
      readlist items from 10 else null
      num.items = dcount(items, @fm)
      item.index = 0
   end

* ======================================================================
* RIGHT - Position at right of index

   public subroutine right
      items = ''
      num.items = 0
      item.index = 0
      setright index.name from fvar
   end
end


